"""Test fixtures for field description extraction tests."""

from tests.unit.utils.fixtures.test_classes_with_docstrings import (
    AllQuoteTypes,
    MixedDocumentation,
    OrderWithMultilineDocstrings,
    PriorityTest,
    ProductWithDocstrings,
    UserWithDocstrings,
)

__all__ = [
    "UserWithDocstrings",
    "ProductWithDocstrings",
    "OrderWithMultilineDocstrings",
    "MixedDocumentation",
    "AllQuoteTypes",
    "PriorityTest",
]
